%tintern
% Plots geometric construction which looks a bit like the main window at
% the (ruined) Tintern Abbey.
%
% h is the height of the centre of the upper cicular window, point (b,0)
% that right arch intersects x axis.
%
% Radius of lower windows (as a fraction of upper radius) is R_over_r
%
% LAST UPDATED by Andy French August 2023

function b = tintern( h, R_over_r, plots )

%Window width
a = 1;

%Fontsize for graphs
fsize = 18;

%

%% GEOMETRIC CALCULATIONS %%

%Calculate radius of upper window
k = sqrt( 1 +( 4*h^2 )/(a^2) ); r = a*( 1 - k/2 );

%Calculate radius of lower circular window
R = R_over_r*r;

%Calculate centre (A,B) of lower window(s)
BB = linspace(-2*a,2*a,1000);
F = bfunc(BB,R,r,a,h);
B = rootfindergeneral(BB,F);
A_R = 0.5*a + sqrt( (r+R)^2 -(h-B).^2 );
A_L = 0.5*a - sqrt( (r+R)^2 -(h-B).^2 );

%Calculate point (b,0) that right arch intersects x axis
i = 0; b = a;
while i==0
    tmax = atan( 2*sqrt( a^2 -0.25*(a+b)^2 )/(a+b) );
    t = linspace( 0, tmax , 1000 );
    x = a*cos(t); x = reflect(x, (a+b)/2 ); y = a*sin(t);
    f = ( x-A_R ).^2 + ( y - B ).^2 - R^2;
    troot = rootfindergeneral(t,f);
    if ~isempty(troot)
        i = 1;
    else
        b = b - 0.001;
    end
end

%b=5*a/7;

%

%% PLOT FIGURE %%
if plots==1
    
    %Plot numeric method for finding B
    figure; set(gca, 'fontsize', fsize); plot( BB,real(F),'linewidth',2);  hold on; grid on;
    xlimits = get(gca,'xlim'); ylimits = get(gca,'ylim');
    plot( [B,B],ylimits ); plot( xlimits,[0,0] );
    title( ['\beta = ',num2str(B) ] ); xlabel('\beta');
    print( gcf, 'tintern_beta_func.png','-dpng','-r300' ); close(gcf);
    
    %Set up figure
    fig = figure('name','tintern window','color',[1 1 1]);
    ax = axes('Nextplot','add'); axis equal; xlim([-0.2*a,1.2*a]); ylim([-a,a]);
    grid on; box on; set(gca,'fontsize',fsize);
    title(['Tintern window. a=',num2str(a),', h=',num2str(h,3),...
        ', r=',num2str(r,3),', R=',num2str(R,3)])
    
    %Left main window arc
    t = linspace(0,60*pi/180,1000);
    x = a - a*cos(t); y = a*sin(t); plot(x,y,'linewidth',2);
    
    %Right main window arc
    t = linspace(0,60*pi/180,1000);
    x = a*cos(t); y = a*sin(t); plot(x,y,'linewidth',2);
    
    %Upper window
    t = linspace(0,2*pi,1000);
    x = r*cos(t) + a/2; y = r*sin(t) + h; plot(x,y,'linewidth',2);
    
    %Plot right and left lower circular windows
    t = linspace(0,2*pi,1000);
    x_L = R*cos(t) + A_L ; x_R = R*cos(t) + A_R ; y = R*sin(t) + B;
    plot(x_L,y,'linewidth',2); plot(x_R,y,'linewidth',2);
    
    %Determine lower arches. For 7 overlapping lower arches, b = 5/7 * a =
    %0.7143*a
    
    %Generate arch coordinates
    tmax = atan( 2*sqrt( a^2 -0.25*(a+b)^2 )/(a+b) );
    t = linspace( 0, tmax , 1000 );
    x = a*cos(t); x = reflect(x, (a+b)/2 ); y = a*sin(t);
    arch_x = [x,fliplr( reflect(x,(a+b)/2) )];
    arch_y = [y,fliplr(y)];
    plot(arch_x,arch_y,'b','linewidth',2);
    
    %Translate and replicate arch
    plot(arch_x-(a-b),arch_y,'b','linewidth',2);
    plot(arch_x-b,arch_y,'b','linewidth',2);
    plot(arch_x+a-2*b,arch_y,'b','linewidth',2);
    
    %Plot vertical columns
    k = (a-b)/2 ;
    plot( [0,0],[-0.75*a,0],'b','linewidth',2);
    plot( k*[1,1],[-0.75*a,0],'b','linewidth',2);
    plot( 2*k*[1,1],[-0.75*a,0],'b','linewidth',2);
    plot( 3*k*[1,1],[-0.75*a,0],'b','linewidth',2);
    plot( 4*k*[1,1],[-0.75*a,0],'b','linewidth',2);
    plot( (a-4*k)*[1,1],[-0.75*a,0],'b','linewidth',2);
    plot( (a-3*k)*[1,1],[-0.75*a,0],'b','linewidth',2);
    plot( (a-2*k)*[1,1],[-0.75*a,0],'b','linewidth',2);
    plot( (a-k)*[1,1],[-0.75*a,0],'b','linewidth',2);
    plot( a*[1,1],[-0.75*a,0],'b','linewidth',2);
    
    %Plot bottom of window
    plot( [0,a],[-0.75*a,-0.75*a],'b','linewidth',2);
    
    %Determine smallest arches
    x = (b+3*a)/4; y = sqrt( a^2 -x^2 ); tmax = atan( y/x );
    t = linspace( 0, tmax , 1000 );
    x = a*cos(t); y = a*sin(t);
    arch_x = [x,fliplr( reflect(x,(3*a+b)/4) )]; arch_y = [y,fliplr(y)];
    plot(arch_x,arch_y,'b','linewidth',2);
    
    %Translate and replicate smallest archs
    plot(arch_x-(a-b)/2,arch_y,'b','linewidth',2);
    plot(arch_x-(a-b),arch_y,'b','linewidth',2);
    plot(arch_x-3*(a-b)/2,arch_y,'b','linewidth',2);
    plot(arch_x-(a+b)/2,arch_y,'b','linewidth',2);
    plot(arch_x-(a+b)/2+(a-b)/2,arch_y,'b','linewidth',2);
    plot(arch_x-(a+b)/2+2*(a-b)/2,arch_y,'b','linewidth',2);
    plot(arch_x-(a+b)/2+3*(a-b)/2,arch_y,'b','linewidth',2);
    
    %Save PNG file and close
    fname = ['a=',n2f(a),' h=',n2f(h),...
        ' r=',n2f(r),' R=',n2f(R),'.png'];
    print( fig, fname,'-dpng','-r300' ); close(fig);
end

%%

%Function to find lower circle y coordinate B
function f = bfunc(B,R,r,a,h)
A = 0.5*a + sqrt( (r+R)^2 -(h-B).^2 );
f = R^2 + A.^2 + B.^2 + 2*R*sqrt( A.^2 + B.^2 ) -a^2;

%%

%Find root(s) of function f(x)
function xroot = rootfindergeneral(x,f)
xroot = [];
for n=1:length(x)-1
    %Check if f(x) has crossed zero
    if ( ( f(n+1)>=0 ) && ( f(n) <=0 ) ) || ( ( f(n+1)<=0 ) && ( f(n) >=0 ) )
        xroot = [xroot,0.5*( x(n) + x(n+1) )];
    end
end

%%

%Refect x coordinates in line x = k
function xx = reflect(x,k)
xx = 2*k - x;

%%

%Number to filename. Decimal points are replaced with a p
function s = n2f(x)
s = strrep( num2str(x,3),'.','p' );

%End of code